﻿using System;
using System.Configuration;
using System.Globalization;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Input;
using Microsoft.IdentityModel.Clients.ActiveDirectory;

namespace GetAirCarrierDelaysClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        //
        // The Tenant is the name of the Azure AD tenant in which this application is registered.
        // The AAD Instance is the instance of Azure, for example public Azure or Azure China.
        // The Client ID is used by the application to uniquely identify itself to Azure AD.
        // The Redirect URI is the URI where Azure AD will return OAuth responses.
        //
        private static string _tenant = ConfigurationManager.AppSettings["aad:Tenant"];
        private static string _aadInstance = ConfigurationManager.AppSettings["aad:AADInstance"];
        private static string _clientId = ConfigurationManager.AppSettings["api:ClientId"];
        private Uri _redirectUri = new Uri(ConfigurationManager.AppSettings["api:RedirectUri"]);

        //
        // To authenticate to the Get Air Carrier API service, the client needs to know the service's App ID URI.
        // To contact the Air Carrier API service we need it's URL as well, so we enter base URL here.
        //
        private static string _apiResourceId = ConfigurationManager.AppSettings["api:ApiResourceId"];
        private static string _apiBaseAddress = ConfigurationManager.AppSettings["api:ApiBaseAddress"];

        // The Authority is the sign-in URL of the tenant.
        private static string _authority = String.Format(CultureInfo.InvariantCulture, _aadInstance, _tenant);

        private HttpClient _httpClient = new HttpClient();
        private AuthenticationContext _authContext = null;

        public MainWindow()
        {
            InitializeComponent();
            
            //
            // As the application starts, try to get an access token from the tokens cache.
            //
            _authContext = new AuthenticationContext(_authority, new FileCache());
            //authContext = new AuthenticationContext(authority); - this is if you do not want to use tokens cache
            try
            {
                _authContext.AcquireToken(_apiResourceId, _clientId, _redirectUri, PromptBehavior.Never);

                // A valid token is in the cache - ready to call our API.
                SignInButton.Content = "Clear Cache";
            }
            catch (AdalException ex)
            {
                if (ex.ErrorCode != "user_interaction_required")
                {
                    // An unexpected error occurred.
                    string message = ex.Message;
                    if (ex.InnerException != null)
                    {
                        message += "Inner Exception : " + ex.InnerException.Message;
                    }
                    MessageBox.Show(message);
                }
                return;
            }
        }

        private async void CallAPIAsync()
        {
            //
            // Get an access token to call the Air Carrier Delay service.
            //
            AuthenticationResult result = null;
            try
            {
                result = _authContext.AcquireToken(_apiResourceId, _clientId, _redirectUri, PromptBehavior.Never);
            }
            catch (AdalException ex)
            {
                // There is no access token in the cache, so prompt the user to sign-in.
                if (ex.ErrorCode == "user_interaction_required")
                {
                    MessageBox.Show("Please sign in first");
                    SignInButton.Content = "Sign In";
                }
                else
                {
                    // An unexpected error occurred.
                    string message = ex.Message;
                    if (ex.InnerException != null)
                    {
                        message += "Inner Exception : " + ex.InnerException.Message;
                    }
                    MessageBox.Show(message);
                }

                return;
            }

            // Once the token has been returned by ADAL, add it to the http authorization header, 
            // before making the call to the actual API service.
            _httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", result.AccessToken);


            // this is where you make the actual API Get Air Delays service call - replace with the call to your own API here
            HttpResponseMessage response;
            try
            {
                Cursor = Cursors.Wait;
                response = await _httpClient.GetAsync(_apiBaseAddress + "/On_Time_Performance");
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
                return;
            }
            finally
            {
                Cursor = Cursors.Arrow;
            }

            if (response.IsSuccessStatusCode)
            {

                // Read the response and databind to the GridView to display To Do items.
                string s = await response.Content.ReadAsStringAsync();
                this.TextBoxResult.Text = s;
            }
            else
            {
                MessageBox.Show("An error occurred : " + response.ReasonPhrase);
            }
        }

        private void CallAPI_Click(object sender, RoutedEventArgs e)
        {
            this.TextBoxResult.Text = string.Empty;
            CallAPIAsync();
        }

        private void SignIn(object sender = null, RoutedEventArgs args = null)
        {
            // If there is already a token in the cache, clear the cache and update the label on the button.
            if (SignInButton.Content.ToString() == "Clear Cache")
            {
                this.TextBoxResult.Text = string.Empty;
                _authContext.TokenCache.Clear();
                // Also clear cookies from the browser control.
                ClearCookies();
                SignInButton.Content = "Sign In";
                return;
            }

            //
            // Get an access token to call Air Carrier service.
            //
            try
            {
                _authContext.AcquireToken(_apiResourceId, _clientId, _redirectUri, PromptBehavior.Always);
                SignInButton.Content = "Clear Cache";
            }
            catch (AdalException ex)
            {
                if (ex.ErrorCode == "authentication_canceled")
                {
                    MessageBox.Show("Sign in was canceled by the user");
                }
                else
                {
                    // An unexpected error occurred.
                    string message = ex.Message;
                    if (ex.InnerException != null)
                    {
                        message += "Inner Exception : " + ex.InnerException.Message;
                    }

                    MessageBox.Show(message);
                }

                return;
            }

        }

        // This function clears cookies from the browser control used by ADAL.
        private void ClearCookies()
        {
            const int INTERNET_OPTION_END_BROWSER_SESSION = 42;
            InternetSetOption(IntPtr.Zero, INTERNET_OPTION_END_BROWSER_SESSION, IntPtr.Zero, 0);
        }

        [DllImport("wininet.dll", SetLastError = true)]
        private static extern bool InternetSetOption(IntPtr hInternet, int dwOption, IntPtr lpBuffer, int lpdwBufferLength);
    }
}